"""
Teleport Restrict v1.0
Written by RideGuy
Idea by XE_Manup

When a restricted class tries to take the teleporter they are slain and respawned
at their teams spawn.  Class restrictions are controlled by a cvar in the config
file teleport_restrict.cfg located in /orangebox/tf/cfg/teleport_restrict.
"""

import es
import gamethread
import usermsg

info = es.AddonInfo()
info.name     = 'Teleport Restrict'
info.version  = '1.0'
info.url      = 'http://forums.mattie.info/cs/forums/viewforum.php?f=45'
info.basename = 'teleport_restrict'
info.author   = 'RideGuy'

es.ServerVar('teleport_restrict', 1.0).makepublic()

classNumbers = {'1':'scout', '2':'sniper', '3':'soldier',
                '4':'demoman', '5':'medic', '6':'heavy',
                '7':'pyro', '8':'spy', '9':'engineer'}

classTracker = {'scout':[], 'sniper':[], 'soldier':[],
                'demoman':[], 'medic':[], 'heavy':[],
                'pyro':[], 'spy':[], 'engineer':[]}

restrictedList = es.ServerVar('tr_restrict', 'scout sniper spy',
                              'Classes restricted from use of teleporter')

tickCheck = es.ServerVar('tr_tick_check', 6,
                         'Frequency in ticks teleporters are checked for restricted travel')

es.server.queuecmd('exec teleport_restrict/teleport_restrict.cfg')

tickCounter = 0

def load():
    es.addons.registerTickListener(tickListener)
    
    for userid in es.getUseridList():
        userid = str(userid)
        classNumber = es.getplayerprop(userid, 'CTFPlayer.m_PlayerClass.m_iClass')
        if classNumber:
            classTracker[classNumbers[str(classNumber)]].append(userid)
    
def unload():
    es.addons.unregisterTickListener(tickListener)

def player_changeclass(event_var):
    userid = event_var['userid']
    classNumber = event_var['class']
    team = event_var['es_userteam']
    
    for playerClass in classTracker:
        if userid in classTracker[playerClass]:
            classTracker[playerClass].remove(userid)
            break
    
    classTracker[classNumbers[classNumber]].append(userid)

def tickListener():
    global tickCounter
    if tickCounter < tickCheck:
        tickCounter += 1
        return
    
    tickCounter = 0
    
    entranceList = es.createentitylist('obj_teleporter_entrance').keys()
    
    for entrance in entranceList:
        if es.getindexprop(entrance, 'CObjectTeleporter.m_iState') == 3:
            checkPlayers()

def checkPlayers():
    for playerClass in str(restrictedList).split(' '):
        for userid in classTracker[playerClass]:
            playerCondition = es.getplayerprop(userid, 'CTFPlayer.m_Shared.m_nPlayerCond')
            if not playerCondition: continue
            
            binaryCondition = getBinary(playerCondition)
            if len(binaryCondition) < 11: continue
            
            if binaryCondition[-11:-10] != '1': continue
            
            punishPlayer(userid, playerClass)

def getBinary(integer):
    '''convert integer to binary string'''
    binaryStr = ''
    if integer < 0:  raise ValueError, "must be a positive integer"
    if integer == 0: return '0'
    while integer > 0:
        binaryStr = str(integer % 2) + binaryStr
        integer = integer >> 1
    return binaryStr

def punishPlayer(userid, playerClass):
    es.sexec(userid, 'kill')
    gamethread.delayed(1, es.spawnplayer,(userid))
    usermsg.centermsg(userid, 'The %s class is not permitted to use the teleporter' % playerClass)